@@ -13,7 +13,7 @@ package PDL::NiceSlice;
 # Modified 2-Oct-2001: don't modify $var(LIST) if it's part of a
 # "for $var(LIST)" or "foreach $var(LIST)" statement.  CED.
 
-$PDL::NiceSlice::VERSION = 0.8;
+$PDL::NiceSlice::VERSION = 0.91;
 
 require PDL::Version; # get PDL version number
 if ("$PDL::Version::VERSION" !~ /cvs$/ and
@@ -205,7 +205,7 @@ sub findslice {
 #  Do final check for "for $avar(LIST)" and "foreach $avar(LIST)" syntax. 
 #  Process into an 'nslice' call only if it's not that.
 
-    if($prefix =~ m/for(each)?\s+\$\w+$/s) {
+    if($prefix =~ m/for(each)?(\s+(my|our))?\s+\$\w+$/s) {
       $processed .= "$prefix".$found;       # foreach statement: Don't translate 
 
     } else {      # statement is a real slice and not a foreach
@@ -270,39 +270,44 @@ sub perldlpp {
  return $new;
 }
 
-# use Damian Conway's simplified source filter interface
-use Filter::Simple sub {
-  # print STDERR "entering Filter...\n";
-  # print STDERR;
-  # unless (m/$prefixpat/) {
-    # print STDERR "got that ->\n";
-    # print STDERR;
-    # print STDERR "leaving prematurely\n";
-    # print $@;
-  #  return;
-  # };
-  $_ = findslice $_;
-  # print STDERR "After\n\n";
-  # print STDERR;
-  # print STDERR "status: $@\n";
-  # print STDERR "leaving Filter...\n";
-};
-# stay clear of __END__ and __DATA__ sections
-# => qr/^\s*(__(END|DATA)__)|(no\s+PDL::NiceSlice\s*;)\s*$/;
+use Filter::Util::Call;
 
-# well it is not quite that simple ;)
-#  since Filter::Simple doesn't have all the bells and whistles yet
-#
-# some symbol table wizardry to get file name and offset
-local $^W = 0; # shut up warnings about redefining import
-my $class = __PACKAGE__;
-my $sub = \&{"${class}::import"}; # get the import meth created by F::Simple
-*{"${class}::import"} = sub {
-  ($file,$offset) = (caller)[1,2];
+sub import {
+  my ($class) = @_;
+  ($file,$offset) = (caller)[1,2];  # for error reporting
   $offset++;
-  # now chain the import sub that was generated by Filter::Simple
-  &$sub(@_);
-};
+  my $terminator = qr/^\s*no\s+$class\s*;\s*(#.*)*$/; # allow trailing comments
+	filter_add(
+		sub {
+			my ($status, $off, $end);
+			my $count = 0;
+			my $data = "";
+			while ($status = filter_read()) {
+				return $status if $status < 0;
+				if ($terminator && m/$terminator/) {
+					$off=1;
+					last;
+				}
+				if (m/^\s*(__END__|__DATA__)\s*$/) {
+				  $end=$1; $off = 1;
+				  last;
+				}
+				$data .= $_;
+				$count++;
+				$_ = "";
+			}
+			$_ = $data;
+			$_ = findslice $_ unless $status < 0; # the actual filter
+			$_ .= "no $class;\n" if $off;
+			$_ .= "$end\n" if $end;
+			return $count;
+		}
+	);
+}
+
+sub unimport {
+  filter_del();
+}
 
 =head1 NAME
 
@@ -327,7 +332,7 @@ PDL::NiceSlice - toward a nicer slicing syntax for PDL
   $a(0:1114;_) .= 0;        # short for $a->flat->(0:1114)
   $b = $a(0:-1:3;|);        # short for $a(0:-1:3)->sever
 
-  # Use with perldl (or include these lines in .perldlrc)
+  # Use with perldl versions < v1.31 (or include these lines in .perldlrc)
   perldl> use PDL::NiceSlice; 
   # next one is required, see below
   perldl> $PERLDL::PREPROCESS = \&PDL::NiceSlice::perldlpp;
@@ -344,8 +349,7 @@ the source filter that provides this new feature.
 
 I<NOTE>: C<PDL::NiceSlice> relies on several modules from CPAN that make
 source filtering and parsing easier. In particular it requires
-F<Filter::Simple> (which in turn requires a module from the F<Filter>
-distribution) and F<Text::Balanced>. To make your life easier it is
+the F<Filter> distribution and F<Text::Balanced>. To make your life easier it is
 recommended that you use the F<CPAN> module to resolve dependencies
 during installation automatically. Using the cpan installation shell
 installation should be as easy as this:
@@ -358,10 +362,10 @@ installation should be as easy as this:
 The new slicing syntax can be switched on and off in scripts
 and perl modules by using or unloading C<PDL::NiceSlice>.
 
-Note: this will I<not> work in the L<perldl|perldl> shell.
+Note: this will I<not> work in the L<perldl|perldl> shell E<lt> v1.31.
 Because the perldl shell uses evals, and NiceSlice is a perl source filter,
 you have to set a special variable to use it within perldl. See below
-how to enable the new slicing syntax within L<perldl|perldl>.
+how to enable the new slicing syntax within older L<perldl|perldl>.
 
 But now back to scripts and modules.
 Everything after C<use PDL::NiceSlice> will be translated
@@ -448,6 +452,10 @@ section below).
 
 =head2 Usage with perldl
 
+I<NOTE>: This information only applies to versions of
+L<perldl|perldl> smaller than 1.31 . From v1.31 onwards
+niceslicing is enabled by default. See L<perldl> for details.
+
 To use the filter in the C<perldl> shell you need to
 add the following two lines to your F<.perldlrc> file:
 
@@ -576,8 +584,8 @@ can write in plain Perl
   $sub = sub { print join ',', @_ };
   $sub->(1,'a');
 
-NOTE: Once C<use PDL::NiceSlice> is in effect (see below how to switch it off
-again in a source file) the source filter will incorrectly
+NOTE: Once C<use PDL::NiceSlice> is in effect (you can always switch it off with
+a line C<no PDL::NiceSlice;> anywhere in the script) the source filter will incorrectly
 replace the above call to C<$sub> with an invocation of the slicing method.
 This is one of the pitfalls of using a source filter that doesn't know
 anything about the runtime type of a variable (cf. the
@@ -895,35 +903,66 @@ be in the next PDL release.
 
 =head1 BUGS
 
-Error checking is probably not yet foolproof.
+=head2 Conditional operator
 
 The conditional operator can't be used in slice expressions (see
 above).
 
+=head2 The C<DATA> file handle
+
+I<Note>: To avoid clobbering the C<DATA> filehandle C<PDL::NiceSlice>
+switches itself off when encountering the C<__END__> or C<__DATA__> tokens.
+This should not be a problem for you unless you use C<SelfLoader> to load
+PDL code including the new slicing from that section. It is even desirable
+when working with L<Inline::Pdlpp|Inline::Pdlpp>, see below.
+
+=head2 Possible interaction with L<Inline::Pdlpp|Inline::Pdlpp>
+
 There is currently an undesired interaction between C<PDL::NiceSlice>
 and the new L<Inline::Pdlpp|Inline::Pdlpp> module (currently only in 
 PDL CVS). Since PP code generally
 contains expressions of the type C<$var()> (to access piddles, etc)
-C<PDL::NiceSlice> recognizes those incorrectly as
-slice expressions and does its substitutions. For the moment
-(until hopefully the parser can deal with that) it is best to
-explicitly switch C<PDL::NiceSlice> off before the section of
-inlined Pdlpp code. For example:
+C<PDL::NiceSlice> recognizes those I<incorrectly> as
+slice expressions and does its substitutions. This is not a problem
+if you use the C<DATA> section for your Pdlpp code -- the recommended
+place for Inline code anyway. In that case
+C<PDL::NiceSlice> will have switched itself off before encountering any
+Pdlpp code (see above):
+
+    # use with Inline modules
+  use PDL;
+  use PDL::NiceSlice;
+  use Inline Pdlpp;
+
+  $a = sequence(10);
+  print $a(0:5);
+
+  __END__
+
+  __Pdlpp__
+
+  ... inline stuff
+
+
+Otherwise switch C<PDL::NiceSlice> explicitly off around the
+Inline::Pdlpp code:
 
   use PDL::NiceSlice;
-  use Inline::Pdlpp;
 
   $a = sequence 10;
   $a(0:3)++;
   $a->inc;
 
-  no PDL::NiceSlice;
+  no PDL::NiceSlice; # switch off before Pdlpp code
+  use Inline Pdlpp => "Pdlpp source code";
 
-  __DATA__
+The cleaner solution is to always stick with the
+C<DATA> way of including your C<Inline> code as
+in the first example. That way you keep your nice Perl
+code at the top and all the ugly Pdlpp stuff etc at
+the bottom.
 
-  __C__
-
-  ppdef (...); # your full pp definition here
+=head2 Bug reports
 
 Feedback and bug reports are welcome. Please include an example
 that demonstrates the problem. Log bug reports in the PDL
